;;;; stack-eggdoc.scm

(use eggdoc)

(define doc '(
	(eggdoc:begin
		(name "stack")
		(description (p "Provides FIFO queue (stack) operations."))
		(author (url "mailto:klovett@pacbell.net" "Kon Lovett"))
		(history (version "1.0" "Initial release"))
		
		(requires
			(url "http://srfi.schemers.org/srfi-1/srfi-1.html" "srfi-1")
			(url "http://www.call-with-current-continuation.org/eggs/syntax-case.html" "syntax-case"))
		
		(usage)
		(download "stack.egg")
		
		(documentation
			(p "stack is a set of procedures and macros supporting FIFO queue
			operations. The stack is treated as an independent object,
			which is modified in place. In opposition to the common
			pattern of a variable being re-assinged a new stack
			representation after each mutating operation.")
			
			(group
						  
				(macro "(make-stack)"
					(p "Returns a new, empty, stack."))

				(macro "(stack? STACK)"
					(p "Returns #t for a stack " (tt "STACK") ", #f
					otherwise. Not fool-proof!"))

				(macro "(stack-empty? STACK)"
					(p "Returns #t for an empty stack " (tt "STACK") ", #f otherwise."))

				(macro "(stack-length STACK)"
					(p "Returns the number of elements on the stack " (tt "STACK") "."))

				(macro "(stack-depth STACK)"
					(p "Returns the index of the last element on the
					stack " (tt "STACK") ", or -1 when stack empty."))

				(macro "(stack-peek STACK [INDEX])"
					(p "Returns the element in stack " (tt "STACK")
					" at index " (tt "INDEX") ". Index must be 0 <= & <=
					(stack-depth), defaults to 0 (top of stack)."))

				(macro "(stack-poke! STACK VALUE [INDEX])"
					(p "Changes the stack " (tt "STACK") " element at index "
					(tt "INDEX") " to value " (tt "VALUE") ". Returns the
					modified stack. The stack is modified in place.
					Index must be 0 <= & <= (stack-depth), defaults to
					0 (top of stack)"))

				(macro "(stack-push! STACK VALUE ...)"
					(p "Pushes value(s) " (tt "VALUE") " onto the stack"
					(tt "STACK") ". Returns the modified stack. The
					stack is modified in place."))

				(macro "(stack-pop! STACK)"
					(p "Removes the top element from the stack " (tt
					"STACK") " and returns it. The stack is modified in
					place."))

				(procedure "(stack-cut! STACK START-DEPTH [END-DEPTH])"
					(p "Removes the stack elements from start-depth "
					(tt "START-DEPTH") " thru end-depth " (tt
					"END-DEPTH") " and returns a list of the stack
					elements. The stack is modified in place. The
					start-depth must be 0 <= & <= (stack-depth). The
					end-depth must be start-depth <= & <=
					(stack-depth), defaults to start-depth."))

				(macro "(list->stack LIST)"
					(p "Returns the list " (tt "LIST") " as a stack. The
					resulting stack may share memory with the list and
					the list should not be modified after this
					operation."))

				(macro "(stack->list STACK)"
					(p "Returns the stack " (tt "STACK") " as a list,
					where the first element of the list is the top
					element of the stack.  The resulting list may share
					memory with the queue object and should not be
					modified."))

				(macro "(stack-for-each Stack PROCEDURE)"
					(p "Applies the procedure " (tt "PROCEDURE") " to
					each element of the stack " (tt "STACK") ", in order
					of top to bottom."))

				(macro "(print-stack STACK)"
					(p "Prints the elements of the stack " (tt "STACK")
					" to the (current-output-port)."))
			)
		)
	)
))

(eggdoc->html doc)
