;;;; stack_ext_1.scm
;;;; Kon Lovett, Mar 29 2005

;; Required by stack.scm

(use srfi-1)

(eval-when (compile)
	(declare
		(unit stack)
		(uses srfi-1)
		(fixnum)
		(disable-interrupts)
		(usual-integrations)
		(bound-to-procedure
			stack-unit:node-ref stack-cut!)
		(export
			 stack-unit:node-ref stack-cut!)
	)
	
	(cond-expand
		[paranoia]
		[else
			(declare
				(no-procedure-checks)
				(no-argc-checks)
				(no-bound-checks)
	)]))

;; Helpers

(define (stack-unit:node-ref loc s n)
	(when (negative? n)
		(error loc "negative stack index" n))
	(let loop ([ns s] [nn n])
		(when (stack-empty? ns)
			(error loc "stack index too large" n))
		(let ([sn (cdr ns)])
			(if (zero? nn)
				sn
				(loop sn (- nn 1))))))

;;;

;; STACK-CUT! Stack Start-Depth !#optional End-Depth
;;
;; Stack - the stack to cut
;;
;; Start-Depth - the depth to start the cut, 0 <= & <= (stack-length
;; Stack) - 1
;;
;; End-Depth - the depth to end the cut, Start-Depth <= & <=
;; (stack-depth Stack) - 1, defaults to Start-Depth
;;
;; Returns a list of the cut stack elements. The stack is modified in place.

(define (stack-cut! s sd #!optional (ed sd))
	(cond
		((negative? sd) (error 'stack-cut! "start depth must be >= 0" sd))
		((negative? ed) (error 'stack-cut! "end depth must be >= 0" ed))
		((< ed sd) (error 'stack-cut! "start depth must be <= to the end depth" sd ed)))
	(let* (	[ds (sub1 sd)]
			[sn (if (negative? ds) s (stack-unit:node-ref 'stack-cut! s ds))]
			[en (stack-unit:node-ref 'stack-cut! sn (- ed sd))]
			[ls (cdr sn)])
		(set-cdr! sn (cdr en))
		(set-cdr! en '())
		ls))
